//=============================================================================
// "Arduino" example program for Crystalfontz ePaper. 
//
// This project is for the CFAP128296C0-0290 :
//
//   https://www.crystalfontz.com/product/cfap128296c00290
//=============================================================================
#include <SPI.h>

#include "ePaper_CFAP128296C00290.h"

//SPI access macros
#define ePaper_RST_0  (digitalWrite(EPD_RESET, LOW))
#define ePaper_RST_1  (digitalWrite(EPD_RESET, HIGH))
#define ePaper_CS_0   (digitalWrite(EPD_CS, LOW))
#define ePaper_CS_1   (digitalWrite(EPD_CS, HIGH))
#define ePaper_DC_0   (digitalWrite(EPD_DC, LOW))
#define ePaper_DC_1   (digitalWrite(EPD_DC, HIGH))
#define ePaper_BS_0   (digitalWrite(EPD_BUSSEL, LOW))
#define ePaper_BS_1   (digitalWrite(EPD_BUSSEL, HIGH))

//=============================================================================
void ePaper_WriteCMD(uint8_t command)
  {
  //Make sure the dsplay is not busy before starting a new command.
  while(0 == digitalRead(EPD_READY));
  //Select the controller   
  ePaper_CS_0;
  //Aim at the command register
  ePaper_DC_0;
  SPI.transfer(command);
  //Deselect the controller
  ePaper_CS_1;
  }
//=============================================================================
// command in data[0]
// parameters in data[1]. . .data[datalen-1]
#define PTR_AND_SIZ(x) x,sizeof(x)
void WriteCMD_StringFlash(const uint8_t *data, uint8_t datalen)
  {
  //Index into *image, that works with pgm_read_byte()
  uint8_t
    index;
  index=0;

  //Make sure the dsplay is not busy before starting a new command.
  while(0 == digitalRead(EPD_READY));
  //Select the controller   
  ePaper_CS_0;
  //The first byte in the string is the command, aim at the command register
  ePaper_DC_0;
  SPI.transfer(pgm_read_byte(&data[index++]));
  //Remember that we wrote the command that was in data[0]
  datalen--;

  //Remaining bytes are data, aim at the data register
  ePaper_DC_1;
  for(uint8_t i= 0;i<datalen;i++)
    {
    SPI.transfer(pgm_read_byte(&data[index++]));
    }
  //Deslect the controller   
  ePaper_CS_1;
  }
//=============================================================================	
void Load_Flash_Image_To_Display_RAM(uint16_t width_pixels,
                                     uint16_t height_pixels,
                                     const char *image)
  {
  //Index into *image, that works with pgm_read_byte()
  uint16_t
    index;
  index=0;
  
  //Get width_bytes from width_pixel, rounding up
  uint8_t
    width_bytes;
  width_bytes=(width_pixels+7)>>3;  
  
  //Make sure the display is not busy before starting a new command.
  while(0 == digitalRead(EPD_READY));
  //Select the controller   
  ePaper_CS_0;
  //Aim at the command register
  ePaper_DC_0;
  //Write the command: DATA START TRANSMISSION 2 (DTM2) (R13H)
  //  Display Start transmission 2
  //  (DTM2, Red Data)
  //
  // This command starts transmitting data and write them into SRAM. To complete
  // data transmission, command DSP (Data transmission Stop) must be issued. Then
  // the chip will start to send data/VCOM for panel.
  //  * In B/W mode, this command writes “NEW” data to SRAM.
  //  * In B/W/Red mode, this command writes “RED” data to SRAM.
  SPI.transfer(0x13);
  //Pump out the "new" data.
  ePaper_DC_1;
  index=0;
  for(uint16_t y=0;y<height_pixels;y++)
    {
    for(uint8_t x=0;x<width_bytes;x++)
      {
      SPI.transfer(pgm_read_byte(&image[index]));
      index++;
      }
    }

  //Aim back at the command register
  ePaper_DC_0;
  //Write the command: DATA STOP (DSP) (R11H)
  SPI.transfer(0x11);
  //Write the command: Display Refresh (DRF)   
  SPI.transfer(0x12);
  //Deslect the controller   
  ePaper_CS_1;
  }	  
//=============================================================================
const uint8_t Power_ON_PON[] PROGMEM =
  {0x04};
const uint8_t Panel_Setting_PSR[] PROGMEM =
  {0x00,0x97};
  // 1001 0111 = Crystalfontz
  // RREB ULDR
  // |||| ||||-- RST_N *1 = run, 0 for software reset
  // |||| |||--- SHD_N *1 = Booster on, 0 = Booster off
  // |||| ||---- SHL   *1 = scan right, 0 = scan left
  // |||| |----- UD     1 = scan up, *0 = scan down
  // ||||------- BWR    0 = Black/White/Red (Runs LU1 & LU2)
  // |||               *1 = Black/White (Runs LU1 only)
  // |||-------- REG_EN 1 = LUT from register, *0 = LUT from OTP
  // ||--------- RES1:RES0 (over-ridden by 0x61)
  //                 00 = 96x230
  //                 01 = 96x252
  //                *10 = 128x296
  //                 11 = 160x296

const uint8_t Resolution_setting_TRES[] PROGMEM =
  {0x61,128,296>>8,296&0xFF};

const uint8_t Vcom_and_data_interval_setting_CDI_On[] PROGMEM =
  {0x50,0x97};
  // 1001 0111 = Crystalfontz
  // BBRB CCCC
  // |||| ||||-- CDI: 0011 = default
  // ||||------- DDX[0]: Black Data Polarity 1=ink, 0=white
  // |||-------- DDX[1]: Red Data Polarity   1=ink, 0=white
  // ||--------- VBD: Border Data Selection
//-----------------------------------------------------------------------------
void ePaper_Init(void)
  {
  //Select 4-wire SPI mode.
  ePaper_BS_0;

  //Give the controller a hardware reset
  ePaper_RST_0;
  delay(1);
  ePaper_RST_1;
  delay(1);

  //Power ON (PON)
  WriteCMD_StringFlash(PTR_AND_SIZ(Power_ON_PON));
  //Panel Setting (PSR)
  WriteCMD_StringFlash(PTR_AND_SIZ(Panel_Setting_PSR));
  //Resolution setting (TRES)
  WriteCMD_StringFlash(PTR_AND_SIZ(Resolution_setting_TRES));
  //Vcom and data interval setting (CDI)
  WriteCMD_StringFlash(PTR_AND_SIZ(Vcom_and_data_interval_setting_CDI_On));
  //From Release of reset to here takes about 37mS
  }
//=============================================================================
const uint8_t Vcom_and_data_interval_setting_CDI_Off[] PROGMEM =
   {0x50,0x17};
const uint8_t VCM_DC_Setting_VDCS_Off[] PROGMEM =
   {0x82,0x00};
const uint8_t Power_Setting_PWR_Off[] PROGMEM =
   {0x01,0x00,0x00,0x00,0x00,0x00};
const uint8_t Power_OFF_POF[] PROGMEM =
   {0x02};
const uint8_t Deep_sleep_DSLP[] PROGMEM =
   {0x07,0xA5};
//-----------------------------------------------------------------------------
void ePaper_PowerOff(void)
  {
  WriteCMD_StringFlash(PTR_AND_SIZ(Vcom_and_data_interval_setting_CDI_Off));
  WriteCMD_StringFlash(PTR_AND_SIZ(VCM_DC_Setting_VDCS_Off));
  WriteCMD_StringFlash(PTR_AND_SIZ(Power_Setting_PWR_Off));
  WriteCMD_StringFlash(PTR_AND_SIZ(Power_OFF_POF));
  WriteCMD_StringFlash(PTR_AND_SIZ(Deep_sleep_DSLP));
  }
//=============================================================================

